package ru.dbotthepony.mc.otm.worldgen

import it.unimi.dsi.fastutil.objects.Reference2ObjectOpenHashMap
import net.minecraft.core.BlockPos
import net.minecraft.util.RandomSource
import net.minecraft.world.level.ChunkPos
import net.minecraft.world.level.WorldGenLevel
import net.minecraft.world.level.chunk.ChunkGenerator
import net.minecraft.world.level.levelgen.feature.FeaturePlaceContext
import net.minecraft.world.level.levelgen.feature.configurations.FeatureConfiguration
import net.minecraft.world.level.levelgen.placement.PlacementContext
import ru.dbotthepony.kommons.util.KOptional
import ru.dbotthepony.mc.otm.worldgen.feature.EnhancedFeature
import java.util.*
import kotlin.collections.HashMap

class EnhancedPlacementContext {
	fun interface Placer {
		fun place(context: EnhancedPlacementContext, positions: List<BlockPos>, feature: EnhancedFeature.Configured<*, *>)
	}

	private data class SharedState(
		val level: WorldGenLevel,
		val generator: ChunkGenerator,
		val random: RandomSource,
		val origin: ChunkPos,
		val placer: Placer,
		val vanillaContext: PlacementContext,
	)

	private val state: SharedState

	val level: WorldGenLevel
		get() = state.level
	val generator: ChunkGenerator
		get() = state.generator
	val random: RandomSource
		get() = state.random
	val origin: ChunkPos
		get() = state.origin
	private val placer: Placer
		get() = state.placer
	val vanillaContext: PlacementContext
		get() = state.vanillaContext

	private val parent: EnhancedPlacementContext?
	private val variables = Reference2ObjectOpenHashMap<PlacementVariable<*>, KOptional<*>>(0)

	private fun <T> recursiveGet(index: PlacementVariable<T>): KOptional<T>? {
		return variables[index] as KOptional<T>? ?: parent?.recursiveGet(index)
	}

	operator fun <T> get(index: PlacementVariable<T>): KOptional<T> {
		return recursiveGet(index) ?: KOptional()
	}

	operator fun <T> set(index: PlacementVariable<T>, value: T) {
		variables[index] = KOptional(value)
	}

	fun <T> remove(index: PlacementVariable<T>): KOptional<T> {
		val old = variables.put(index, KOptional<T>())
		return old as KOptional<T>? ?: KOptional()
	}

	fun remove(index: Collection<PlacementVariable<*>>) {
		index.forEach { remove(it) }
	}

	constructor(
		level: WorldGenLevel,
		generator: ChunkGenerator,
		random: RandomSource,
		origin: ChunkPos,
		placer: Placer,
	) {
		state = SharedState(
			level = level,
			generator = generator,
			random = random,
			origin = origin,
			placer = placer,
			vanillaContext = PlacementContext(level, generator, Optional.empty())
		)

		parent = null
	}

	private constructor(parent: EnhancedPlacementContext) {
		this.state = parent.state
		this.parent = parent
	}

	fun push(): EnhancedPlacementContext {
		return EnhancedPlacementContext(this)
	}

	fun place(positions: List<BlockPos>, feature: EnhancedFeature.Configured<*, *>) {
		placer.place(this, positions, feature)
	}

	fun <FC : FeatureConfiguration> vanillaFeatureContext(config: FC, position: BlockPos): FeaturePlaceContext<FC> {
		return FeaturePlaceContext(Optional.empty(), level, generator, random, position, config)
	}
}
