package ru.dbotthepony.mc.otm.worldgen.placement

import com.mojang.serialization.Codec
import com.mojang.serialization.MapCodec
import net.minecraft.core.BlockPos
import net.minecraft.world.level.levelgen.placement.PlacementModifier
import net.neoforged.bus.api.IEventBus
import ru.dbotthepony.mc.otm.registry.MBuiltInRegistries
import ru.dbotthepony.mc.otm.registry.MDeferredRegister
import ru.dbotthepony.mc.otm.registry.MRegistries
import ru.dbotthepony.mc.otm.worldgen.EnhancedPlacementContext
import java.util.stream.Collectors

interface EnhancedPlacementModifier {
	interface Type<T : EnhancedPlacementModifier> {
		val codec: MapCodec<T>
	}

	fun evaluate(context: EnhancedPlacementContext, positions: List<BlockPos>): List<BlockPos>
	val type: Type<*>

	class Wrapper(val parent: PlacementModifier) : EnhancedPlacementModifier {
		override fun evaluate(context: EnhancedPlacementContext, positions: List<BlockPos>): List<BlockPos> {
			return positions.stream()
				.flatMap { parent.getPositions(context.vanillaContext, context.random, it) }
				// use Red-Black tree set instead of AVL tree set because we are write-intense
				.collect(Collectors.toCollection(::ArrayList))
		}

		override val type: Type<*>
			get() = Companion

		companion object : Type<Wrapper> {
			override val codec: MapCodec<Wrapper> = PlacementModifier.CODEC.xmap(::Wrapper, Wrapper::parent).fieldOf("parent")
		}
	}

	object Passthrough : EnhancedPlacementModifier, Type<Passthrough> {
		override fun evaluate(context: EnhancedPlacementContext, positions: List<BlockPos>): List<BlockPos> {
			return positions
		}

		override val type: Type<*>
			get() = this

		override val codec: MapCodec<Passthrough> = MapCodec.unit(this)
	}

	companion object {
		private val registrar = MDeferredRegister(MRegistries.PLACEMENT_MODIFIER)

		init {
			registrar.register("wrapper") { Wrapper.Companion }
			registrar.register("passthrough") { Passthrough }
		}

		val CODEC: Codec<EnhancedPlacementModifier> by lazy {
			MBuiltInRegistries.PLACEMENT_MODIFIER.byNameCodec().dispatch({ it.type }, { it.codec })
		}

		internal fun register(bus: IEventBus) {
			registrar.register(bus)
		}
	}
}
